header {
/*
 [The "BSD licence"]
 Copyright (c) 2005-2006 Terence Parr
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.antlr_yggdrasil.generated;
import org.antlr_yggdrasil.tool.*;
import org.antlr_yggdrasil.tool.Grammar;
import java.util.*;
import java.io.*;
import org.antlr_yggdrasil.analysis.*;
import org.antlr_yggdrasil.misc.*;
import antlr.*;
}

/** Read in an ANTLR grammar and build an AST.  Try not to do
 *  any actions, just build the tree.
 *
 *  The phases are:
 *
 *		antlr.g (this file)
 *		assign.types.g
 *		define.g
 *		buildnfa.g
 *		antlr.print.g (optional)
 *		codegen.g
 *
 *  Terence Parr
 *  University of San Francisco
 *  2005
 */
class YggdrasilParser extends Parser;
options {
	buildAST = true;
	exportVocab=Yggdrasil;
	ASTLabelType="GrammarAST";
	k=3;
}

tokens {
	OPTIONS="options";
	TOKENS="tokens";
	PARSER="parser";
	
	ASSIGN_ATTRIBUTE;
	LABEL_ATTRIBUTE;
    LEXER;
    RULE;
    BLOCK;
    OPTIONAL;
    CLOSURE;
    POSITIVE_CLOSURE;
    SYNPRED;
    RANGE;
    CHAR_RANGE;
    EPSILON;
    ALT;
    EOR;
    EOB;
    EOA; // end of alt
    CHARSET;
    SET;
    ID;
    LEXER_GRAMMAR;
    PARSER_GRAMMAR;
    TREE_GRAMMAR;
    COMBINED_GRAMMAR;
    SYN_SEMPRED; // (...) =>   it's a synpred converted to sempred
    FRAGMENT="fragment";
    BANGEDUP;
}

{
	org.antlr_yggdrasil.tool.Grammar grammar = null;
	protected int gtype = 0;
	protected String currentRuleName = null;
	protected GrammarAST currentBlockAST = null;

	/* this next stuff supports construction of the Tokens artificial rule.
	   I hate having some partial functionality here, I like doing everything
	   in future tree passes, but the Tokens rule is sensitive to filter mode.
	   And if it adds syn preds, future tree passes will need to process the
	   fragments defined in Tokens; a cyclic dependency.
	   As of 1-17-06 then, Tokens is created for lexer grammars in the
	   antlr grammar parser itself.

	   This grammar is also sensitive to the backtrack grammar option that
	   tells ANTLR to automatically backtrack when it can't compute a DFA.

	   7-2-06 I moved all option processing to antlr.g from define.g as I
	   need backtrack option etc... for blocks.  Got messy.
	*/
	protected List lexerRuleNames = new ArrayList();
	public List getLexerRuleNames() { return lexerRuleNames; }

	public void setGrammar(org.antlr_yggdrasil.tool.Grammar g, int type) {
		grammar = g;
		gtype = type;
	}

	protected GrammarAST setToBlockWithSet(GrammarAST b) {
		GrammarAST alt = #(#[ALT,"ALT"],#b,#[EOA,"<end-of-alt>"]);
		prefixWithSynPred(alt);
		return #(#[BLOCK,"BLOCK"],
		           alt,
		           #[EOB,"<end-of-block>"]
		        );
	}

	protected GrammarAST createBlockFromDupAlt(GrammarAST alt) {
		GrammarAST nalt = (GrammarAST)astFactory.dupTree(alt);
		GrammarAST blk = #(#[BLOCK,"BLOCK"],
						   nalt,
						   #[EOB,"<end-of-block>"]
						  );
		return blk;
	}

	/** Rewrite alt to have a synpred as first element;
	 *  (xxx)=>xxx
	 *  but only if they didn't specify one manually.
	 */
	protected void prefixWithSynPred(GrammarAST alt) {
		// if they want backtracking and it's not a lexer rule in combined grammar
		String autoBacktrack = (String)currentBlockAST.getOption("backtrack");
		if ( autoBacktrack==null ) {
			autoBacktrack = (String)grammar.getOption("backtrack");
		}
		if ( autoBacktrack!=null&&autoBacktrack.equals("true") &&
			 !(gtype==COMBINED_GRAMMAR &&
			 Character.isUpperCase(currentRuleName.charAt(0))) &&
			 alt.getFirstChild().getType()!=SYN_SEMPRED )
		{
			// duplicate alt and make a synpred block around that dup'd alt
			GrammarAST synpredBlockAST = createBlockFromDupAlt(alt);

			// Create a SYN_SEMPRED node as if user had typed this in
			// Effectively we replace (xxx)=>xxx with {synpredxxx}? xxx
			GrammarAST synpredAST = createSynSemPredFromBlock(synpredBlockAST);

			// insert SYN_SEMPRED as first element of alt
			synpredAST.getLastSibling().setNextSibling(alt.getFirstChild());
			alt.setFirstChild(synpredAST);
		}
	}

	protected GrammarAST createSynSemPredFromBlock(GrammarAST synpredBlockAST) {
		// add grammar fragment to a list so we can make fake rules for them
		// later.
		String predName = grammar.defineSyntacticPredicate(synpredBlockAST,currentRuleName);
		// convert (alpha)=> into {synpredN}? where N is some pred count
		// during code gen we convert to function call with templates
		String synpredinvoke = predName;
		GrammarAST p = #[SYN_SEMPRED,synpredinvoke];
		p.setEnclosingRule(currentRuleName);
		// track how many decisions have synpreds
		grammar.blocksWithSynPreds.add(currentBlockAST);
		return p;
	}

	public GrammarAST createSimpleRuleAST(String name,
										  GrammarAST block,
										  boolean fragment)
   {
   		GrammarAST modifier = null;
   		if ( fragment ) {
   			modifier = #[FRAGMENT,"fragment"];
   		}
   		GrammarAST EORAST = #[EOR,"<end-of-rule>"];
   		GrammarAST EOBAST = block.getLastChild();
		EORAST.setLine(EOBAST.getLine());
		EORAST.setColumn(EOBAST.getColumn());
		GrammarAST ruleAST =
		   #([RULE,"rule"],
                 [ID,name],modifier, block,EORAST);
		ruleAST.setLine(block.getLine());
		ruleAST.setColumn(block.getColumn());
		return ruleAST;
	}

    public void reportError(RecognitionException ex) {
		Token token = null;
		try {
			token = LT(1);
		}
		catch (TokenStreamException tse) {
			ErrorManager.internalError("can't get token???", tse);
		}
		ErrorManager.syntaxError(
			ErrorManager.MSG_SYNTAX_ERROR,
			token,
			"antlr: "+ex.toString(),
			ex);
    }

    public void cleanup(GrammarAST root) {
		if ( gtype==LEXER_GRAMMAR ) {
			String filter = (String)grammar.getOption("filter");
			GrammarAST tokensRuleAST =
			    grammar.addArtificialMatchTokensRule(
			    	root,
			    	lexerRuleNames,
			    	filter!=null&&filter.equals("true"));
		}
    }
}

file[org.antlr_yggdrasil.tool.Grammar g]
	:
	(
		fileheader
	)*
	(
		typeDecls
	)?
	(
		optionsSpec
	)?
	grammar[g]
	;


grammar![org.antlr_yggdrasil.tool.Grammar g]
{
	this.grammar = g;
	GrammarAST opt=null;
	Token optionsStartToken = null;
	Map opts;
}
	:
	( cmt:DOC_COMMENT  )?
        gr:grammarType gid:id SEMI
	(
		gattr:grammarAttributeDecls
	)?
	(	{optionsStartToken=LT(1);}
		opts=optionsSpec {grammar.setOptions(opts, optionsStartToken);}
		{opt=(GrammarAST)returnAST;}
	)?
	(ts:tokensSpec!)?
	( a:action )?
	r:rules
        EOF
        {
        #grammar = #(null, #(#gr, #gid, #cmt, #gattr, #opt, #ts, null, #a, #r));
        cleanup((GrammarAST) #grammar);
        }
	;

fileheader
	:
	(
		id
	)?
	"header"^
	(	
		LCURLY
		( optionsSpec )?
		( tokensSpec )?
		( action )?
		RCURLY!
	)
	;


grammarType
	:
	(	"lexer"!  {gtype=LEXER_GRAMMAR;}    // pure lexer
	|	"parser"! {gtype=PARSER_GRAMMAR;}   // pure parser
    	|	"tree"!   {gtype=TREE_GRAMMAR;}     // a tree parser
    	|		{gtype=COMBINED_GRAMMAR;}   // merged parser/lexer
	)
    	gr:"grammar" {#gr.setType(gtype);}
    ;


// Attribute specification
grammarAttributeDecls
	:
	ATTRIBUTE_STMT^
	( grammarAttributeDecl )*
	RCURLY!
	;

grammarAttributeDecl
	:
	( "public" )?		// allows external access
	attributeVarDecl
	|
		"import" STRING_LITERAL	// external declaration file
	;


ruleAttributeDecls
	:
	ATTRIBUTE_STMT^
	(
	 	attributeVarDecl
		|
			attributeUseDecl
	)*
	RCURLY!
	;


typeDecls
	:
	ATTRIBUTE_STMT^
	(
		attributeTypeDecl
	)*
	RCURLY!
	;
	

attributeTypeDecl
	:
	( "native" )?
	id
	id
	LCURLY^
	(
	 	attributeVarDecl
	)*
	RCURLY!
	|
		"native"
		"atomic"^
		id
	;

attributeVarDecl
	:
	id
	(	OPEN_ELEMENT_OPTION
		id ( COMMA! id )*
		CLOSE_ELEMENT_OPTION
	)?
	id
	SEMI^
	;

idQualifier
	:
	DOT_TEXT
	|
		arrayItem
	;

arrayItem
	:
	(	LBRACKET^
		(	STRING_LITERAL
			|
				id (idQualifier)*
		)
		RBRACKET!
	)
	;


attributeUseDecl
	:
	(	id 
		ASSIGN
		( id (idQualifier)* )
		|
			"new" id
		|
			"using" id
	)
	SEMI!
	;	
// END ATTRIBUTE DECLARATION and TYPE DEFINITION

optionsSpec returns [Map opts=new HashMap()]
	:
	OPTIONS^ (option[opts] SEMI!)+ RCURLY!
	;

option[Map opts]
{
    Object value=null;
}
    :   o:id ASSIGN^ value=optionValue
    	{
    	opts.put(#o.getText(), value);
    	}
    	/*
    	{
    	if ( #o.getText().equals("filter") && #v.getText().equals("true") ) {
    		isFilterMode = true;
    	}
    	else if ( #o.getText().equals("backtrack") && #v.getText().equals("true") ) {
    		if ( currentRuleName==null ) { // must grammar level
    			isAutoBacktrackMode = true;
    		}
    		else {
    			blockAutoBacktrackMode = true;
    		}
    	}
    	}
    	*/
    ;

optionValue returns [Object value=null]
    :   x:id			 {value = #x.getText();}
    |   s:STRING_LITERAL {String vs = #s.getText();
                          value=vs.substring(1,vs.length()-1);}
    |   c:KEYWORD   {String vs = #c.getText();
                          value=vs.substring(1,vs.length()-1);}
    |   i:INT            {value = new Integer(#i.getText());}
    |	ss:STAR			 {#ss.setType(STRING_LITERAL); value = "*";} // used for k=*
//  |   cs:charSet       {value = #cs;} // return set AST in this case
    ;

/*
optionValue
	:	id
	|   STRING_LITERAL
	|	KEYWORD
	|	INT
//	|   cs:charSet       {value = #cs;} // return set AST in this case
	;
*/

/*
will probably need for char vocab spec later
charSet
	:   LPAREN^ {#LPAREN.setType(CHARSET);}
	        charSetElement ( OR^ charSetElement )*
	    RPAREN!
	;

charSetElement
	:   c1:KEYWORD
	|   c2:KEYWORD RANGE^ c3:KEYWORD
	;
*/

tokensSpec
	:	TOKENS^
			( tokenSpec	)+
		RCURLY!
	;

tokenSpec
	:
	TOKEN_REF
	(	OPEN_ELEMENT_OPTION	// Payload type spec
		id
		CLOSE_ELEMENT_OPTION!
	)?
	(
		ASSIGN^
		KEYWORD
	)?
	SEMI!
	;


rules
	:  
	(
		rule
		|
			constructorRule
	)+
    ;

rule!
{
GrammarAST modifier=null, blk=null, blkRoot=null, eob=null;
int start = ((TokenWithIndex)LT(1)).getIndex();
int startLine = LT(1).getLine();
GrammarAST opt = null;
Map opts = null;
}
	:
	(	d:DOC_COMMENT	
	)?
	(	p1:"protected"	{modifier=#p1;}
	|	p2:"public"	{modifier=#p2;}
	|	p3:"private"    {modifier=#p3;}
	|	p4:"fragment"	{modifier=#p4;}
	)?
	ruleName:id
	{	currentRuleName=#ruleName.getText();
		if ( gtype==LEXER_GRAMMAR && #p4==null ) {
         		lexerRuleNames.add(currentRuleName);
		}
	}
	( BANG )?
	( throwsSpec )?
	( opts=optionsSpec {opt=(GrammarAST)returnAST;} )?
	(
		ra:ruleAttributeDecls
	)?
	( act:action )?
	colon:COLON
	{
		blkRoot = #[BLOCK,"BLOCK"];
		blkRoot.setLine(colon.getLine());
		blkRoot.setColumn(colon.getColumn());
		eob = #[EOB,"<end-of-block>"];
	}
	(	(setNoParens SEMI) => s:setNoParens // try to collapse sets
		{
			blk = #(blkRoot,#(#[ALT,"ALT"],#s,#[EOA,"<end-of-alt>"]),eob);
		}
	|	b:altList[opts] {blk = #b;}
	)
	semi:SEMI
	( ex:exceptionGroup )?
    {
    int stop = ((TokenWithIndex)LT(1)).getIndex()-1; // point at the semi or exception thingie
	eob.setLine(semi.getLine());
	eob.setColumn(semi.getColumn());
    GrammarAST eor = #[EOR,"<end-of-rule>"];
   	eor.setEnclosingRule(#ruleName.getText());
	eor.setLine(semi.getLine());
	eor.setColumn(semi.getColumn());
	GrammarAST root = #[RULE,"rule"];
	root.ruleStartTokenIndex = start;
	root.ruleStopTokenIndex = stop;
	root.setLine(startLine);
	root.setOptions(opts);
    #rule = #(root,
              #ruleName,modifier, opt, #ra, #act,blk,ex,eor);
    }
	;


// "Tree" construction rule
constructorRule
	:
	RULE_REF
	ATTRIBUTE_COLON^
	( attributeItem )+
	SEMI!
	;


throwsSpec
	:	"throws" id ( COMMA id )*
		
	;

attributeDecls
	:
	ATTRIBUTE_STMT^
	( attributeDecl)*
	RCURLY
	;

attributeDecl
	:
	(	"uses"
	|	id
	)
	id
	SEMI^
	;

/** Build #(BLOCK ( #(ALT ...) EOB )+ ) */
block
{
	GrammarAST save = currentBlockAST;
	Map opts=null;
}
	:
	(set) => s:set  // special block like ('a'|'b'|'0'..'9')
	|
		lp:LPAREN^ {#lp.setType(BLOCK); #lp.setText("BLOCK");}
		(
			// 2nd alt and optional branch ambig due to
			// linear approx LL(2) issue.  COLON ACTION
			// matched correctly in 2nd alt.
			options {
				warnWhenFollowAmbig = false;
			}
		:
			(opts=optionsSpec {#block.setOptions(grammar,opts);})?
			COLON!
		|	action COLON!
		)?

		{currentBlockAST = #lp;}

		a1:alternative 
		{prefixWithSynPred(#a1);}
		(	OR! a2:alternative
			{if (LA(1)==OR||LA(2)==QUESTION) prefixWithSynPred(#a2);}
		)*

        rp:RPAREN!
        {
		currentBlockAST = save;
        GrammarAST eob = #[EOB,"<end-of-block>"];
        eob.setLine(rp.getLine());
        eob.setColumn(rp.getColumn());
        #block.addChild(eob);
        }
    ;

altList[Map opts]
{
	GrammarAST blkRoot = #[BLOCK,"BLOCK"];
	blkRoot.setOptions(opts);
	blkRoot.setLine(LT(1).getLine());
	blkRoot.setColumn(LT(1).getColumn());
	GrammarAST save = currentBlockAST;
	currentBlockAST = #blkRoot;
}
	:
	a1:alternative
    	{prefixWithSynPred(#a1);}
    	(	OR! a2:alternative
		{if (LA(1)==OR) prefixWithSynPred(#a2);}
	)*
        {
        #altList = #(blkRoot,#altList,#[EOB,"<end-of-block>"]);
        currentBlockAST = save;
        }
    ;

alternative
{
    GrammarAST eoa = #[EOA, "<end-of-alt>"];
    GrammarAST altRoot = #[ALT,"ALT"];
    altRoot.setLine(LT(1).getLine());
    altRoot.setColumn(LT(1).getColumn());
}
    :   ( el:element )+ ( exceptionSpecNoLabel! )?
        {
            if ( #alternative==null ) {
                #alternative = #(altRoot,#[EPSILON,"epsilon"],eoa);
            }
            else {
            	// we have a real list of stuff
               	#alternative = #(altRoot, #alternative, eoa);
            }
        }
    |   {
    	GrammarAST eps = #[EPSILON,"epsilon"];
		eps.setLine(LT(0).getLine()); // get line/col of '|' or ':' (prev token)
		eps.setColumn(LT(0).getColumn());
    	#alternative = #(altRoot,eps,eoa);
    	}
	;

exceptionGroup
	:
	(
		exceptionSpec
	)+
	;

exceptionSpec
	:
	"exception"^
	(
		arg_decl
	)?
	(
		exceptionHandler
	)*
	;

arg_decl
	:
	LBRACKET!
	(	STRING_LITERAL			// anonymous StringTemplate
		|
			id		// StringTemplate name
	)
	( 	COLON
		( assignment )+
	)?
	RBRACKET^
	;
	

exceptionSpecNoLabel
	:
	"exception"^
	(
		exceptionHandler
	)*
	;

exceptionHandler
	:
	"catch"^
	arg_decl
	action
	;

element
	:
	(	
		label
		elementItem
		|
			action
	)
	(
		optionsSpec
	)?
	;


// Templated action
action
	:
	LCURLY!
	(	STRING_LITERAL			// anonymous StringTemplate
		|
			id		// StringTemplate name
	)
	( 	COLON!
		( assignment )+
	)?
	(	RCURLY^
		|
			SEMPRED^
	)
	;


assignment
	:
	(	t:TOKEN_REF^		{ #t.setType(ID); }
		|
			r:RULE_REF^	{ #r.setType(ID); }
	)
	ASSIGN!
	(	attribute
		|
			TOKEN_REF	// symbolic constants
		|
			STRING_LITERAL
	)
	SEMI!
	;


attrLabel
	:
	(	// assign but do not remove AST
		a:ATTRIBUTE  { #a.setType(ASSIGN_ATTRIBUTE); }
	|	// assign and remove AST
		i:RULE_REF	{ #i.setType(LABEL_ATTRIBUTE); }
	)
	( idQualifier )*
	ASSIGN!
	;

label
	:
	(	(attrLabel)=> attrLabel
		|
	)
	;

elementItem
{ GrammarAST subrule=null; }
	:
	(
		r:RULE_REF
    		(	subrule=ebnfSuffix[#r] {#elementItem=subrule;}
			( b:BANG^ { #b.setType(BANGEDUP); } )?
			| BANG
		)?
		|
			range
		|
			terminal
    			( ast_suffix )?
		|
			n:notSet
        		(	subrule=ebnfSuffix[#n] {#elementItem = subrule;}
				( b1:BANG^ { #b1.setType(BANGEDUP); } )?
			|
    				( ast_suffix )?
			)
		        {	int line = LT(1).getLine();
				int col = LT(1).getColumn();

				#n.setLine(line);
				#n.setColumn(col);
			}
		|
			ebnf
		|
			attributeItem
		|
			treeMatcher
	)
	;

attribute
	:
	ATTRIBUTE
	( idQualifier )*
	;


// ATTRIBUTE syntax
attributeGroup
	:
	ATTRIBUTE_GROUP^
	(
		attributeItem
		|
			STRING_LITERAL
	)*
	RPAREN!
	;

	
attributeBlock
	:
	action
	(	(
			attributeItem
		)*
	)
	OR attributeBlock
	|
		(
			attributeItem
		)*
	;

attributeItem
	:
	label
	attributeElement
	;

attributeElement
	:
	attribute
	( CARET )?
	|
		attributeGroup
		( CARET )?
	|
		ATTRIBUTE_STMT^
		attributeBlock
		RCURLY!
	|
		LPAREN
		(attributeItem)*
		RPAREN
	|
		treeConstructor
	|
		CURRENT_TREE
		( BANG )?
	;

treeConstructor
	:
	TREE_CONSTRUCTOR^
	attributeItem
	(
		attributeItem
	)+
	RPAREN!
	;
// END ATTRIBUTE syntax

notSet
{
    GrammarAST subrule=null;
}
	:
	n:NOT^
	(	notTerminal
        |   // special case: single element is not a set
		(LPAREN setElement RPAREN)=> LPAREN! setElement RPAREN! 
        |
		set
	)
	;

/** Match two or more set elements */
set
	:
	LPAREN! s:setNoParens RPAREN!
    ;

setNoParens
{Token startingToken = LT(1);}
	:
	setElement (OR! setElement)+
        {
        	GrammarAST ast = new GrammarAST();
		ast.initialize(new TokenWithIndex(SET, "SET"));
		((TokenWithIndex)ast.token)
			.setIndex(((TokenWithIndex)startingToken).getIndex());
		#setNoParens = #(ast, #setNoParens);
        }
	;

setElement
	:
	KEYWORD
	|
		{gtype!=LEXER_GRAMMAR}? TOKEN_REF
	|
		{gtype!=LEXER_GRAMMAR}? STRING_LITERAL
	|
		range
	;

treeMatcher
	:
	TREE_BEGIN^
	rootNode
	(
		element
	)+
	RPAREN!
	;

rootNode
	:
	(attrLabel)=> attrLabel terminal
	|
		terminal ( ast_suffix )?
	;

/** matches ENBF blocks (and sets via block rule) */
ebnf
{
    int line = LT(1).getLine();
    int col = LT(1).getColumn();
}
	:
	b:block
	(	(	QUESTION!    {if (#b.getType()==SET) #b=setToBlockWithSet(#b);
						 #ebnf=#([OPTIONAL,"?"],#b);}
		|	STAR!	    {if (#b.getType()==SET) #b=setToBlockWithSet(#b);
						 #ebnf=#([CLOSURE,"*"],#b);}
		|	PLUS!	    {if (#b.getType()==SET) #b=setToBlockWithSet(#b);
						 #ebnf=#([POSITIVE_CLOSURE,"+"],#b);}
		)
		( ba:BANG^  { #ba.setType(BANGEDUP); } )?
	|
		IMPLIES! // syntactic predicate
		{
			if ( gtype==COMBINED_GRAMMAR &&
			     Character.isUpperCase(currentRuleName.charAt(0)) )
			{
		    		#ebnf = #(#[SYNPRED,"=>"],#b);
				// ignore for lexer rules in combined
		 	}
			else {
		    		#ebnf = createSynSemPredFromBlock(#b);
			}
		}
	|
		{#ebnf = #b;}
	)
	{	((GrammarAST) #ebnf).setLine(line);
		((GrammarAST) #ebnf).setColumn(col);}
	;

range!
{
GrammarAST subrule=null, root=null;
}
	:
	c1:KEYWORD RANGE c2:KEYWORD
	{
		GrammarAST r = #[CHAR_RANGE,".."];
		r.setLine(c1.getLine());
		r.setColumn(c1.getColumn());
		#range = #(r, #c1, #c2);
		root = (GrammarAST) #range;
	}
    	(subrule=ebnfSuffix[root] {#range=subrule;})?
	;

terminal
{
GrammarAST ebnfRoot=null, subrule=null;
}
	:
	cl:KEYWORD
    		( subrule=ebnfSuffix[#cl] {#terminal=subrule;} )?

	|
		tr:TOKEN_REF
    		( subrule=ebnfSuffix[#tr] {#terminal=subrule;} )?
		// Args are only valid for lexer rules
	|
		sl:STRING_LITERAL
		// rewrite as sempred
		{ if ((gtype != LEXER_GRAMMAR)
			|| ((gtype == COMBINED_GRAMMAR) 
			&& Character.isUpperCase(currentRuleName.charAt(0))))
			#terminal = #( null, #(#[SEMPRED, "}?"],
				#[ID, "stringPred"],
				#(#[ID, "string"], sl)),
				#[WILDCARD, "."]);
		}
    		( subrule=ebnfSuffix[#sl] {#terminal=subrule;} )?

	|
		wi:WILDCARD
    		( subrule=ebnfSuffix[#wi] {#terminal=subrule;} )?
	;

ast_suffix
	:
	CARET
	|	BANG
	;

ebnfSuffix[GrammarAST elemAST] returns [GrammarAST subrule=null]
{
GrammarAST ebnfRoot=null;
}
	:!
	(	QUESTION {ebnfRoot = #[OPTIONAL,"?"];}
   	|	STAR     {ebnfRoot = #[CLOSURE,"*"];}
   	|	PLUS     {ebnfRoot = #[POSITIVE_CLOSURE,"+"];}
   	)
    	{
       		ebnfRoot.setLine(elemAST.getLine());
		ebnfRoot.setColumn(elemAST.getColumn());
		GrammarAST blkRoot = #[BLOCK,"BLOCK"];
		GrammarAST eob = #[EOB,"<end-of-block>"];
		GrammarAST save = currentBlockAST;
		eob.setLine(elemAST.getLine());
		eob.setColumn(elemAST.getColumn());
		GrammarAST alt = #(#[ALT,"ALT"],elemAST,#[EOA,"<end-of-alt>"]);
		prefixWithSynPred(alt);
  		subrule =
			#(ebnfRoot,
			#(blkRoot,alt,eob)
			);
			currentBlockAST = save;
	}
	;

notTerminal
	:
	cl:KEYWORD
	|
		tr:TOKEN_REF

	|
		STRING_LITERAL
	;

idList
	:
		(id)+
	;

id
	:
	TOKEN_REF {#id.setType(ID);}
	|
		RULE_REF  {#id.setType(ID);}
	;

/** Match anything that looks like an ID and return tree as token type ID */
idToken
	:
	TOKEN_REF 		{#idToken.setType(ID);}
	|	RULE_REF	{#idToken.setType(ID);}
	;


class YggdrasilLexer extends Lexer;
options {
	k=2;
	exportVocab=Yggdrasil;
	testLiterals=false;
	interactive=true;
	charVocabulary='\003'..'\377';
}

{
    /** advance the current column number by one; don't do tabs.
     *  we want char position in line to be sent to AntlrWorks.
     */
    public void tab() {
		setColumn( getColumn()+1 );
    }
}


WS	:	(	' '
		|	'\t'
		|	('\r')? '\n' {newline();}
		)
	;

COMMENT :
	( SL_COMMENT | t:ML_COMMENT {$setType(t.getType());} )
	;

protected
SL_COMMENT
 	:	"//"
 	 	(	(" $ANTLR")=> " $ANTLR " SRC ('\r')? '\n' // src directive
 		|	( options {greedy=false;} : . )* ('\r')? '\n'
		)
		{ newline(); }
	;

protected
ML_COMMENT :
	"/*"
	(	{ LA(2)!='/' }? '*' {$setType(DOC_COMMENT);}
	|
	)
	(
		options {
			greedy=false;  // make it exit upon "*/"
		}
	:	'\r' '\n'	{newline();}
	|	'\n'		{newline();}
	|	~('\n'|'\r')
	)*
	"*/"
	;

OPEN_ELEMENT_OPTION
	:	'<'
	;

CLOSE_ELEMENT_OPTION
	:	'>'
	;

COMMA : ',';

QUESTION :	'?' ;

ATTRIBUTE_PRED: "?{" ;

ATTRIBUTE :
	'@'
	INTERNAL_TEXT_REF
	;

ATTRIBUTE_COLON : "@:" ;

ATTRIBUTE_GROUP : "@(" ;

ATTRIBUTE_STMT : "@{" ;

CURRENT_TREE : "@$" ;

TREE_BEGIN : "#(" ;

TREE_CONSTRUCTOR : "^(" ;

LPAREN:	'(' ;

RPAREN:	')' ;

LBRACKET : '[' ;

RBRACKET : ']' ;

COLON :	':' ;

STAR:	'*' ;

PLUS:	'+' ;

ASSIGN : '=' ;

IMPLIES : "=>" ;

SEMI:	';' ;

CARET: '^' ;

BANG : '!' ;

OR	:	'|' ;

protected
WILDCARD :  ;

RANGE : ".." ;

NOT :	'~' ;

LCURLY: '{' ;

RCURLY
	:
	'}'
	( '?'! {_ttype = SEMPRED;} )?
	;

KEYWORD
	:	'\'' (ESC|~'\'')+ '\''
	;

STRING_LITERAL
	:
	'"' (ESC|~'"')* '"'
	;


protected
ESC	:	'\\'
		(	'n'
		|	'r'
		|	't'
		|	'b'
		|	'f'
		|	'"'
		|	'\''
		|	'\\'
		|	'>'
		|	('0'..'3')
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:
			('0'..'9')
				(
					options {
						warnWhenFollowAmbig = false;
					}
				:
				'0'..'9'
				)?
			)?
		|	('4'..'7')
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:
			('0'..'9')
			)?
		|	'u' XDIGIT XDIGIT XDIGIT XDIGIT
		|	. // unknown, leave as it is
		)
	;

protected
DIGIT
	:	'0'..'9'
	;

protected
XDIGIT :
		'0' .. '9'
	|	'a' .. 'f'
	|	'A' .. 'F'
	;

INT	:	('0'..'9')+
	;

protected
ACTION_STRING_LITERAL
	:	'"' (ACTION_ESC|'\n'{newline();}|~'"')* '"'
	;

protected
ACTION_ESC
	:	"\\'"
	|	"\\\""
	|	'\\' ~('\''|'"')
	;

DOT_TEXT
	:
	'.'
	(	INTERNAL_TEXT_REF
	|
		{ _ttype = WILDCARD; }
	)
	;

TOKEN_REF
options { testLiterals = true; }
	:	'A'..'Z'
		(	// scarf as many letters/numbers as you can
			options {
				warnWhenFollowAmbig=false;
			}
		:
			'a'..'z'|'A'..'Z'|'_'|'0'..'9'
		)*
	;

// we get a warning here when looking for options '{', but it works right
RULE_REF
options { testLiterals = true; }
{
	int t=0;
}
	:	t=INTERNAL_RULE_REF {_ttype=t;}
		(	{t==OPTIONS}? WS_LOOP ('{' {_ttype = OPTIONS;})?
		|	{t==TOKENS}? WS_LOOP ('{' {_ttype = TOKENS;})?
		|
		)
	;

protected
WS_LOOP
	:	(	// grab as much WS as you can
			options {
				greedy=true;
			}
		:
			WS
		|	COMMENT
		)*
	;

protected
INTERNAL_RULE_REF returns [int t]
{
	t = RULE_REF;
}
	:	'a'..'z'
		(	// scarf as many letters/numbers as you can
			options {
				warnWhenFollowAmbig=false;
			}
		:
			'a'..'z'|'A'..'Z'|'_'|'0'..'9'
		)*
		{	if (t == RULE_REF)
				t = testLiteralsTable(t);
		}
	;

protected
WS_OPT
	:	(WS)?
	;

/** Reset the file and line information; useful when the grammar
 *  has been generated so that errors are shown relative to the
 *  original file like the old C preprocessor used to do.
 */
protected
SRC	:	"src" ' ' file:ACTION_STRING_LITERAL ' ' line:INT
		{
		newline();
		setFilename(file.getText().substring(1,file.getText().length()-1));
		setLine(Integer.parseInt(line.getText()));
		$setType(Token.SKIP); // don't let this go to the parser
		}
	;

protected
INTERNAL_TEXT_REF
	:	(	// scarf as many letters/numbers as you can
			options {
				warnWhenFollowAmbig=false;
			}
		:
			'a' .. 'z' | '_')
			('a'..'z'|'A'..'Z'|'_'|'0'..'9')*
	;


