header {
/*
 [The "BSD licence"]
 Copyright (c) 2005-2006 Terence Parr
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
	package org.antlr_yggdrasil.generated;
	import org.antlr_yggdrasil.tool.*;
	import org.antlr_yggdrasil.tool.Grammar;
	import java.util.*;
	import org.antlr_yggdrasil.analysis.*;
	import java.io.*;
	import org.antlr_yggdrasil.tool.ErrorManager;
	import org.antlr_yggdrasil.tool.GrammarAST;
	import org.antlr_yggdrasil.runtime.*;
}

/** [Warning: TJP says that this is probably out of date as of 11/19/2005,
 *   but since it's probably still useful, I'll leave in.  Don't have energy
 *   to update at the moment.]
 *
 *  Compute the token types for all literals and rules etc..  There are
 *  a few different cases to consider for grammar types and a few situations
 *  within.
 *
 *  CASE 1 : pure parser grammar
 *	a) Any reference to a token gets a token type.
 *  b) The tokens section may alias a token name to a string or char
 *
 *  CASE 2 : pure lexer grammar
 *  a) Import token vocabulary if available. Set token types for any new tokens
 *     to values above last imported token type
 *  b) token rule definitions get token types if not already defined
 *  c) literals do NOT get token types
 *
 *  CASE 3 : merged parser / lexer grammar
 *	a) Any char or string literal gets a token type in a parser rule
 *  b) Any reference to a token gets a token type if not referencing
 *     a fragment lexer rule
 *  c) The tokens section may alias a token name to a string or char
 *     which must add a rule to the lexer
 *  d) token rule definitions get token types if not already defined
 *  e) token rule definitions may also alias a token name to a literal.
 *     E.g., Rule 'FOR : "for";' will alias FOR to "for" in the sense that
 *     references to either in the parser grammar will yield the token type
 *
 *  What this pass does:
 *
 *  0. Collects basic info about the grammar like grammar name and type;
 *     Oh, I have go get the options in case they affect the token types.
 *     E.g., tokenVocab option.
 *     Imports any token vocab name/type pairs into a local hashtable.
 *  1. Finds a list of all literals and token names.
 *  2. Finds a list of all token name rule definitions;
 *     no token rules implies pure parser.
 *  3. Finds a list of all simple token rule defs of form "<NAME> : <literal>;"
 *     and aliases them.
 *  4. Walks token names table and assign types to any unassigned
 *  5. Walks aliases and assign types to referenced literals
 *  6. Walks literals, assigning types if untyped
 *  4. Informs the Grammar object of the type definitions such as:
 *     g.defineToken(<charliteral>, ttype);
 *     g.defineToken(<stringliteral>, ttype);
 *     g.defineToken(<tokenID>, ttype);
 *     where some of the ttype values will be the same for aliases tokens.
 */
class YggdrasilAssignTokenTypes extends TreeParser;

options {
	importVocab = Yggdrasil;
	ASTLabelType = "GrammarAST";
    codeGenBitsetTestThreshold=999;
}

{
    public void reportError(RecognitionException ex) {
		Token token = null;
		if ( ex instanceof MismatchedTokenException ) {
			token = ((MismatchedTokenException)ex).token;
		}
		else if ( ex instanceof NoViableAltException ) {
			token = ((NoViableAltException)ex).token;
		}
        ErrorManager.syntaxError(
            ErrorManager.MSG_SYNTAX_ERROR,
            token,
            "assign.types: "+ex.toString(),
            ex);
    }

protected Grammar grammar;
protected Map stringLiterals = new LinkedHashMap(); // Map<literal,Integer>
protected Map tokens = new LinkedHashMap();         // Map<name,Integer>
protected TypeMap types = new TypeMap();
/** Track actual lexer rule defs so we don't get repeated token defs in 
 *  generated lexer.
 */
protected Set tokenRuleDefs = new HashSet();        // Set<name>
protected Map aliases = new LinkedHashMap();        // Map<name,literal>
protected String currentRuleName;
protected static final Integer UNASSIGNED = new Integer(-1);
protected static final Integer UNASSIGNED_IN_PARSER_RULE = new Integer(-2);

/** Track string literals in any non-lexer rule (could be in tokens{} section) */

public TypeMap getTypes() {
	return types;
}

protected void trackString(GrammarAST t) {
	// if lexer, don't allow aliasing in tokens section
	if ( currentRuleName==null && grammar.type==Grammar.LEXER ) {
		ErrorManager.grammarError(ErrorManager.MSG_CANNOT_ALIAS_TOKENS_IN_LEXER,
								  grammar,
								  t.token,
								  t.getText());
		return;
	}
	// if not in a combined grammar rule or lexer rule, cannot reference literals
	// (unless defined previously via tokenVocab option)
	if ( grammar.type!=Grammar.COMBINED && grammar.type!=Grammar.LEXER &&
	     grammar.getTokenType(t.getText())==Label.INVALID )
    {
		ErrorManager.grammarError(ErrorManager.MSG_LITERAL_NOT_ASSOCIATED_WITH_LEXER_RULE,
								  grammar,
								  t.token,
								  t.getText());
	}
	// otherwise add literal to token types if referenced from parser rule
	// or in the tokens{} section
	if ( (currentRuleName==null ||
         Character.isLowerCase(currentRuleName.charAt(0))) &&
         grammar.getTokenType(t.getText())==Label.INVALID )
	{
		stringLiterals.put(t.getText(), UNASSIGNED_IN_PARSER_RULE);
	}
}

protected void trackToken(GrammarAST t) {
	// imported token names might exist, only add if new
	if ( grammar.getTokenType(t.getText())==Label.INVALID ) {
		tokens.put(t.getText(), UNASSIGNED);
	}
}

protected void trackTokenRule(GrammarAST t,
							  GrammarAST modifier,
							  GrammarAST block)
{
	// imported token names might exist, only add if new
	if ( grammar.type==Grammar.LEXER || grammar.type==Grammar.COMBINED ) {
		if ( !Character.isUpperCase(t.getText().charAt(0)) ) {
			return;
		}
		int existing = grammar.getTokenType(t.getText());
		if ( existing==Label.INVALID ) {
			tokens.put(t.getText(), UNASSIGNED);
		}
		// look for "<TOKEN> : <literal> ;" pattern
		GrammarAST stringAlias = #(#[BLOCK], #(#[ALT], #[STRING_LITERAL]));
		GrammarAST charAlias = #(#[BLOCK], #(#[ALT], #[KEYWORD]));
		if ( matchesStructure(block,charAlias) ||
             matchesStructure(block,stringAlias) )
		{
			alias(t, (GrammarAST)block.getFirstChild().getFirstChild());
			tokenRuleDefs.add(t.getText());
		}
	}
	// else error
}

protected boolean matchesStructure(AST a, AST b) {
	// the empty tree is always a subset of any tree.
	if (b == null || a==null) {
		return true;
	}

	// check roots first.
	if ( a.getType()!=b.getType() ) {
		return false;
	}

	// if roots match, do full list partial match test on children.
	if (a.getFirstChild() != null) {
		if (!matchesStructure(a.getFirstChild(),b.getFirstChild())) {
			return false;
		}
	}
	return true;
}

protected void alias(GrammarAST t, GrammarAST s) {
	aliases.put(t.getText(), s.getText());
}

protected void assignTypes() {
	/*
	System.out.println("stringLiterals="+stringLiterals);
	System.out.println("tokens="+tokens);
	System.out.println("aliases="+aliases);
	*/

	assignTokenIDTypes();

	aliasTokenIDsAndLiterals();

	assignStringTypes();

	/*
	System.out.println("AFTER:");
	System.out.println("stringLiterals="+stringLiterals);
	System.out.println("tokens="+tokens);
	System.out.println("aliases="+aliases);
	*/

	notifyGrammarObject();
}

	protected void assignStringTypes() {
		// walk string literals assigning types to unassigned ones
		Set s = stringLiterals.keySet();
		for (Iterator it = s.iterator(); it.hasNext();) {
			String lit = (String) it.next();
			Integer oldTypeI = (Integer)stringLiterals.get(lit);
			int oldType = oldTypeI.intValue();
			if ( oldType<Label.MIN_TOKEN_TYPE ) {
				Integer typeI = new Integer(grammar.getNewTokenType());
				stringLiterals.put(lit, typeI);
				// if string referenced in combined grammar parser rule,
				// automatically define in the generated lexer
				grammar.defineLexerRuleForStringLiteral(lit, typeI.intValue());
			}
		}
	}

	protected void aliasTokenIDsAndLiterals() {
		if ( grammar.type==Grammar.LEXER ) {
			return; // strings/chars are never token types in LEXER
		}
		// walk aliases if any and assign types to aliased literals if literal
		// was referenced
		Set s = aliases.keySet();
		for (Iterator it = s.iterator(); it.hasNext();) {
			String tokenID = (String) it.next();
			String literal = (String)aliases.get(tokenID);
			if ( literal.charAt(0)==Grammar.QUOTE_CHAR && stringLiterals.get(literal)!=null ) {
				stringLiterals.put(literal, tokens.get(tokenID));
				// an alias still means you need a lexer rule for it
				Integer typeI = (Integer)tokens.get(tokenID);
				if ( !tokenRuleDefs.contains(tokenID) ) {
					grammar.defineLexerRuleForAliasedStringLiteral(tokenID, literal, typeI.intValue());
				}
			}
		}
	}

	protected void assignTokenIDTypes() {
		// walk token names, assigning values if unassigned
		Set s = tokens.keySet();
		for (Iterator it = s.iterator(); it.hasNext();) {
			String tokenID = (String) it.next();
			if ( tokens.get(tokenID)==UNASSIGNED ) {
				tokens.put(tokenID, new Integer(grammar.getNewTokenType()));
			}
		}
	}

	protected void notifyGrammarObject() {
		Set s = tokens.keySet();
		for (Iterator it = s.iterator(); it.hasNext();) {
			String tokenID = (String) it.next();
			int ttype = ((Integer)tokens.get(tokenID)).intValue();
			grammar.defineToken(tokenID, ttype);
		}
		s = stringLiterals.keySet();
		for (Iterator it = s.iterator(); it.hasNext();) {
			String lit = (String) it.next();
			int ttype = ((Integer)stringLiterals.get(lit)).intValue();
			grammar.defineToken(lit, ttype);
		}
	}

	protected void init(Grammar g) {
		this.grammar = g;
		g.setTypes(types);
		g.initTypeMap();
	}
}



file[Grammar g]
{
	init(g);
}
	:
	(
		fileheader
	)*
	(
		typeDecls
	)?
	(
		optionsSpec
	)?
	grammar[g]
	;


fileheader
	:
	#(	"header"
		( ID )?
		LCURLY
		( optionsSpec )?
		( tokensSpec )?
		( action )?
	)
	;


// Attribute specification syntax
grammarAttributeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			grammarAttributeDecl
		)*
	)
	;


grammarAttributeDecl
	:
	(
		"public"
	)?
	attributeVarDecl[null]
	|
		"import"
		STRING
	;


ruleAttributeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			attributeVarDecl[null]
			|
				attributeUseDecl
		)*
	)
	;


typeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			attributeTypeDecl
		)*
	)
	;
	

attributeTypeDecl
{	TypeInfo info = null;
	boolean isNative = false;
}
	:
	#( 
		"atomic"
		"native"
		i:ID
		{	info = new TypeInfo(#i.getText());
			info.setNative(true);
			info.setAtomic(true);
			types.addType(info);
		}
	)
	|
		#( 
			LCURLY
			(
				"native"
				{ isNative = true; }
			)?
			i1:ID
			i2:ID
			{	TypeInfo parent = types.getInfo(#i1.getText());
				info = new TypeInfo(#i2.getText(), parent);
				info.setArrayed(parent.isArrayed());
		
				if (isNative)
					info.setNative(true);
			}
			(
				attributeVarDecl[info]
			)*
			{ types.addType(info); }
		)
	;

attributeVarDecl [TypeInfo info ]
{	ArrayList typeArgs = null;
	String type = null;
}
	:
	#( 
		SEMI
		t:ID
		(	OPEN_ELEMENT_OPTION
			{ typeArgs = new ArrayList(); }
			(	data:ID
				{ typeArgs.add(#data.getText()); }
			)+
			CLOSE_ELEMENT_OPTION
		)?
		i:ID
		{	if (typeArgs == null)
				type = #t.getText();
			else
				type = types.addGenericType(#t.getText(),
						typeArgs);

			if (info != null)
				info.addField(type, #i.getText());
			else
				types.addVariable(#i.getText(), type);
		}
	)
	;


idQualifier
	:
	DOT_TEXT
	( arrayQualifier )?
	;

arrayQualifier
	:
	#(	LBRACKET
		(	STRING_LITERAL
			|
				ID
				( arrayQualifier )?
				(idQualifier)*
		)
	)
	;


attributeUseDecl
	:
	ID
	ASSIGN
	(
		ID ( arrayQualifier )?
		( idQualifier )*
	)
	|
		"new"
		ID
	|
		"using"
		ID
	;
// END ATTRIBUTE DECLARATION and TYPE DEFINITION


grammar[Grammar g]
	:
	(
		#(	LEXER_GRAMMAR
			{grammar.type = Grammar.LEXER;}
		  	grammarSpec
		)
		|
			#(	PARSER_GRAMMAR
				{grammar.type = Grammar.PARSER;}
				grammarSpec
			)
		|
			#(	TREE_GRAMMAR
				{grammar.type = Grammar.TREE_PARSER;}
				grammarSpec
			)
		|
			#(	COMBINED_GRAMMAR
				{grammar.type = Grammar.COMBINED;}
				grammarSpec
			)
	)
        {assignTypes();}
	;

grammarSpec
{Map opts=null;}
	:
	id:ID {grammar.setName(#id.getText());}
	(cmt:DOC_COMMENT)?
	(
		grammarAttributeDecls
	)?
	(optionsSpec)?
        (tokensSpec)?
        rules
	;

optionsSpec returns [Map opts=new HashMap()]
    :   #( OPTIONS (option[opts])+ )
    ;

option[Map opts]
{
    String key=null;
    Object value=null;
}
    :   #( ASSIGN id:ID {key=#id.getText();} value=optionValue )
        {
        opts.put(key,value);
        // check for grammar-level option to import vocabulary
        if ( currentRuleName==null && key.equals("tokenVocab") ) {
            grammar.importTokenVocabulary((String)value);
        }
        }
    ;

optionValue returns [Object value=null]
    :   id:ID			 {value = #id.getText();}
    |   s:STRING_LITERAL {value = #s.getText();}
    |   c:KEYWORD   {value = #c.getText();}
    |   i:INT            {value = new Integer(#i.getText());}
//  |   cs:charSet       {value = #cs;} // return set AST in this case
    ;

charSet
	:   #( CHARSET charSetElement )
	;

charSetElement
	:   c:KEYWORD
	|   #( OR c1:KEYWORD c2:KEYWORD )
	|   #( RANGE c3:KEYWORD c4:KEYWORD )
	;

tokensSpec
	:	#( TOKENS ( tokenSpec )+ )
	;

tokenSpec
	:
	t:TOKEN_REF           {trackToken(t);}
	(	OPEN_ELEMENT_OPTION
		i:ID
	)?
	|
		#(	ASSIGN
			t2:TOKEN_REF       {trackToken(t2);}
			( OPEN_ELEMENT_OPTION ID )?
			c:KEYWORD   {trackString(c); alias(t2,c);}
		 )
	;


rules
	:
	(
		rule
		|
			constructorRule
	)+
	;


rule
	:
	#(	RULE id:ID {currentRuleName=#id.getText();}
		(m:modifier)?
		(optionsSpec)?
		( ruleAttributeDecls )?
		( action )?
		b:block
		(exceptionGroup)?
		EOR
		{trackTokenRule(#id,#m,#b);}
         )
    ;


constructorRule
	:
	#(	ATTRIBUTE_COLON
		RULE_REF
		( attributeItem )+
	)
	;


modifier
	:	"protected"
	|	"public"
	|	"private"
	|	"fragment"
	;

block
    :   #(  BLOCK
            (optionsSpec)?
            ( alternative )+
            EOB   
         )
    ;

alternative
    :   #( ALT (element)+ EOA )
    ;

exceptionGroup
	:	( exceptionSpec )+
    ;

exceptionSpec
    :   #("exception" ( arg_decl )? ( exceptionHandler )*)
    ;


arg_decl
	:
	#(	RBRACKET
		(	STRING			// anonymous StringTemplate
			|
				ID		// StringTemplate name
		)
		( 	COLON
			( assignment )+
		)?
	)
	;
	

exceptionHandler
    :    #("catch" arg_decl action)
    ;

element
	:
	(	atom
		|	#(NOT atom)
		|	#(RANGE atom atom)
		|	#(CHAR_RANGE atom atom)
		|	ebnf
		|	tree
	)
	( ast_suffix )?
	|	attrLabel
		(	atom
			|
				#(NOT atom)
			|
				ebnf
			|
				tree
			|
				attributeElement
		)
	|	attributeElement
	|	#( SYNPRED block ) 
	|	action
	|	sempred
	|	SYN_SEMPRED
	|	EPSILON 
	;

// Templated action
action
	:
	#(	RCURLY
		template
	)
	;

sempred
	:
	#(	SEMPRED
		template
	)
	;

template
	:
	(	STRING			// anonymous StringTemplate
		|
			ID		// StringTemplate name
	)
	(
		( assignment )+
	)?
	;


assignment
	:
	#(	ID
		(	attribute
			|
				TOKEN_REF	// symbolic constants
			|
				STRING_LITERAL
		)
	)
	;



attrLabel
	:
	(	// assign but do not remove AST
		a:LABEL_ATTRIBUTE
	|	// assign and remove AST
		ASSIGN_ATTRIBUTE
	)
	( arrayQualifier )?
	(
		idQualifier
	)*
	;


ebnf
	:
	#( BANGEDUP baseebnf )
	|
		baseebnf
	;

baseebnf
	:
	block
	|
		#( OPTIONAL block ) 
	|
		#( CLOSURE block )  
	|
		#( POSITIVE_CLOSURE block ) 
	;

tree:   #(TREE_BEGIN  element (element)*  )
    ;


attribute
	:
	ATTRIBUTE
	( arrayQualifier )?
	( idQualifier )*
	;



atom
    :   RULE_REF
    |   t:TOKEN_REF      {trackToken(t);}
    |   c:KEYWORD   {trackString(c);}
    |   s:STRING_LITERAL
    |   WILDCARD
    |	set
    ;

ast_suffix
	:	CARET
	|	BANG
	;

set :   #(SET (setElement)+ )
    ;

setElement
    :   c:KEYWORD   {trackString(c);}
    |   t:TOKEN_REF      {trackToken(t);}
    |   s:STRING_LITERAL
    |	#(CHAR_RANGE c1:KEYWORD c2:KEYWORD)
    	{
    	trackString(c1);
    	trackString(c2);
    	}
    ;

// ATTRIBUTE syntax
attributeGroup
	:
	#( 
		ATTRIBUTE_GROUP
		(
			attributeItem
			|
				STRING_LITERAL
		)*
	)
	;

attributeBlock
	:
	(
		attributeItem
	)*
	|
		sempred
		(
			attributeItem
		)*
		( 
			OR
			attributeBlock
		)
	;


attributeStatements
	:
	#(	ATTRIBUTE_STMT
		attributeBlock
	)
	;

attributeItem
	:
	( attrLabel)?
	attributeElement
	;

attributeElement
	:
	attribute	
	(
		CARET
	)?
	|
		attributeGroup
		( CARET )?
	|
		attributeStatements
	|
		CURRENT_TREE
		( BANG )?
	|
		LPAREN
		(
			attributeItem
		)*
		RPAREN
	|
		treeConstructor
	;


treeConstructor
	:
	#(
		TREE_CONSTRUCTOR
		attributeItem
		(
			attributeItem
		)+
	)
	;
// END ATTRIBUTE syntax

