header {
/*
 [The "BSD licence"]
 Copyright (c) 2005-2006 Terence Parr
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.antlr_yggdrasil.generated;
import org.antlr_yggdrasil.tool.*;
import org.antlr_yggdrasil.tool.Grammar;
import java.util.*;
import org.antlr_yggdrasil.analysis.*;
import org.antlr_yggdrasil.misc.*;
import org.antlr.stringtemplate.*;
}

/** Build an NFA from a tree representing an ANTLR grammar. */
class YggdrasilTreeToNFAConverter extends TreeParser;

options {
	importVocab = Yggdrasil;
	ASTLabelType = "GrammarAST";
}

{
/** Factory used to create nodes and submachines */
protected NFAFactory factory = null;

/** Which NFA object are we filling in? */
protected NFA nfa = null;

/** Which grammar are we converting an NFA for? */
protected Grammar grammar = null;

protected String currentRuleName = null;

public YggdrasilTreeToNFAConverter(Grammar g, NFA nfa, NFAFactory factory) {
	this();
	this.grammar = g;
	this.nfa = nfa;
	this.factory = factory;
}

/* These shouldn't be here, but ... */
protected StringTemplateGroup templates;
protected StringTemplateGroup userTemplates;

public void setTemplates(StringTemplateGroup t, StringTemplateGroup u) {
	templates = t;
	userTemplates = u;
}

protected String getUname(String nm) {
	String firstChar = nm.substring(0,1).toUpperCase();
	String uname = firstChar + nm.substring(1);

	return uname;
}

protected void init() {
    // define all the rule begin/end NFAStates to solve forward reference issues
    Collection rules = grammar.getRules();
    for (Iterator itr = rules.iterator(); itr.hasNext();) {
		Rule r = (Rule) itr.next();
        String ruleName = r.name;
        NFAState ruleBeginState = factory.newState();
        ruleBeginState.setDescription("rule "+ruleName+" start");
		ruleBeginState.setEnclosingRuleName(ruleName);
        grammar.setRuleStartState(ruleName, ruleBeginState);
        NFAState ruleEndState = factory.newState();
        ruleEndState.setDescription("rule "+ruleName+" end");
        ruleEndState.setAcceptState(true);
		ruleEndState.setEnclosingRuleName(ruleName);
        grammar.setRuleStopState(ruleName, ruleEndState);
    }
}

protected void addFollowTransition(String ruleName, NFAState following) {
     //System.out.println("adding follow link to rule "+ruleName);
     // find last link in FOLLOW chain emanating from rule
     NFAState end = grammar.getRuleStopState(ruleName);
     while ( end.transition(1)!=null ) {
         end = (NFAState)end.transition(1).target;
     }
     if ( end.transition(0)!=null ) {
         // already points to a following node
         // gotta add another node to keep edges to a max of 2
         NFAState n = factory.newState();
         Transition e = new Transition(Label.EPSILON, n);
         end.addTransition(e);
         end = n;
     }
     Transition followEdge = new Transition(Label.EPSILON, following);
     end.addTransition(followEdge);
}

protected void finish() {
    List rules = new LinkedList();
    rules.addAll(grammar.getRules());
    int numEntryPoints = factory.build_EOFStates(rules);
    if ( numEntryPoints==0 ) {
        ErrorManager.grammarError(ErrorManager.MSG_NO_GRAMMAR_START_RULE,
                                  grammar,
                                  null,
                                  grammar.getName());
    }
}

    public void reportError(RecognitionException ex) {
		Token token = null;
		if ( ex instanceof MismatchedTokenException ) {
			token = ((MismatchedTokenException)ex).token;
		}
		else if ( ex instanceof NoViableAltException ) {
			token = ((NoViableAltException)ex).token;
		}
        ErrorManager.syntaxError(
            ErrorManager.MSG_SYNTAX_ERROR,
            token,
            "buildnfa: "+ex.toString(),
            ex);
    }
}


file
	:
	(
		fileheader
	)*
	(
		typeDecls
	)?
	( optionsSpec )?
	grammar
	;


fileheader
	:
	#(	"header"
		( ID )?
		LCURLY
		( optionsSpec )?
		( tokensSpec )?
		( action )?
	)
	;


optionsSpec 
	:
	#( OPTIONS (option)+ )
	;

option
	:
	#( ASSIGN ID optionValue )
	;

optionValue
	:
	ID
	|	STRING_LITERAL
	|	KEYWORD
	|	INT
	;

tokensSpec
	:
	#( TOKENS ( tokenSpec )+ )
	;

tokenSpec
	:
	TOKEN_REF
	(	OPEN_ELEMENT_OPTION
		ID
	)?
	|
		#(	ASSIGN
			TOKEN_REF
			( OPEN_ELEMENT_OPTION ID )?
			c:KEYWORD
		 )
	;


// Attribute specification syntax
grammarAttributeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			grammarAttributeDecl
		)*
	)
	;

grammarAttributeDecl
	:
	(
		"public"
	)?
	attributeVarDecl
	|
		"import"
		STRING_LITERAL
	;

ruleAttributeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			attributeVarDecl
			|
				attributeUseDecl
		)*
	)
	;


typeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			attributeTypeDecl
		)*
	)
	;
	

attributeTypeDecl
	:
	#( 
		"atomic"
		"native"
		ID
	)
	|
		#( 
			LCURLY
			(
				"native"
			)?
			ID
			ID
			(
				attributeVarDecl
			)*
		)
	;

attributeVarDecl
	:
	#( 
		SEMI
		ID
		(	OPEN_ELEMENT_OPTION
			ID ( ID )*
			CLOSE_ELEMENT_OPTION
		)?
		ID
	)
	;


idQualifier
	:
	DOT_TEXT
	( arrayQualifier )?
	;

arrayQualifier
	:
	#(	LBRACKET
		(	STRING_LITERAL
			|
				ID
				( arrayQualifier )?
				(idQualifier)*
		)
	)
	;


attributeUseDecl
	:
	ID
	ASSIGN
	(
		ID
		( arrayQualifier )?
		( idQualifier )*
	)
	|
		"new"
		ID
	|
		"using"
		ID
	;
// END ATTRIBUTE DECLARATION and TYPE DEFINITION


grammar
    :   {init();}
        ( #( LEXER_GRAMMAR grammarSpec )
	    | #( PARSER_GRAMMAR grammarSpec )
	    | #( TREE_GRAMMAR grammarSpec )
	    | #( COMBINED_GRAMMAR grammarSpec )
	    )
        {finish();}
    ;

grammarSpec
	:
	ID
	(cmt:DOC_COMMENT)?
	(
		grammarAttributeDecls
	)?
        ( optionsSpec )?
        ( tokensSpec )?
        rules
	;

rules
	:
	(
		rule
	)+
	;


rule
{
    StateCluster g=null;
    StateCluster b = null;
    String r=null;
}
    :   #( RULE id:ID {r=#id.getText();}
		{currentRuleName = r; factory.setCurrentRuleName(r);}
		(modifier)?
		( optionsSpec )?
		( ruleAttributeDecls )?
		( action )?
           #(BLOCK b=block EOB)
           (exceptionGroup)?
           EOR
           {
           /* 11/28/2005: removed to treat Tokens rule like any other
           if ( r.equals(Grammar.ARTIFICIAL_TOKENS_RULENAME) ) {
                NFAState ruleState = factory.build_ArtificialMatchTokensRuleNFA();
                if ( grammar.getNumberOfAltsForDecisionNFA(ruleState)>1 ) {
	                ruleState.setDecisionASTNode(#BLOCK); // always track ast node
                    int d = grammar.assignDecisionNumber( ruleState );
                    grammar.setDecisionNFA( d, ruleState );
                    grammar.setDecisionBlockAST(d, #BLOCK);
                }
                // hook rule start state for Tokens to its manually-created start
                NFAState start = grammar.getRuleStartState(r);
		        start.addTransition(new Transition(Label.EPSILON, ruleState));
           }
           else */
           {
				if ( Character.isLowerCase(r.charAt(0)) ||
					 grammar.type==Grammar.LEXER )
				{
					// attach start node to block for this rule
					NFAState start = grammar.getRuleStartState(r);
					start.setAssociatedASTNode(#id);
					start.addTransition(new Transition(Label.EPSILON, b.left));

					// track decision if > 1 alts
					if ( grammar.getNumberOfAltsForDecisionNFA(b.left)>1 ) {
						b.left.setDescription(grammar.grammarTreeToString((GrammarAST) #rule));
						b.left.setDecisionASTNode((GrammarAST) #BLOCK);
						int d = grammar.assignDecisionNumber( b.left );
						grammar.setDecisionNFA( d, b.left );
                    	grammar.setDecisionBlockAST(d, #BLOCK);
					}

					// hook to end of rule node
					NFAState end = grammar.getRuleStopState(r);
					b.right.addTransition(new Transition(Label.EPSILON,end));
				}
           }
           }
         )
    ;


constructorRule
	:
	#(	ATTRIBUTE_COLON
		RULE_REF
		( attributeItem )+
	)
	;


modifier
	:	"protected"
	|	"public"
	|	"private"
	|	"fragment"
	;

block returns [StateCluster g = null]
{
    StateCluster a = null;
    List alts = new LinkedList();
}
    :   ( optionsSpec )? // ignore
		( a=alternative {alts.add(a);} )+
        {
        g = factory.build_AlternativeBlock(alts);
        }
    ;

alternative returns [StateCluster g=null]
{
    StateCluster e = null;
}
    :   #( ALT (e=element {g = factory.build_AB(g,e);} )+ EOA )
        {
        if (g==null) { // if alt was a list of actions or whatever
            g = factory.build_Epsilon();
        }
        else {
        	factory.optimizeAlternative(g);
        }
        }
    ;

exceptionGroup
	:	( exceptionSpec )+
    ;

exceptionSpec
    :   #("exception" ( arg_decl )? ( exceptionHandler )*)
    ;

arg_decl
	:
	#(	RBRACKET
		(	STRING_LITERAL		// anonymous StringTemplate
			|
				ID		// StringTemplate name
		)
		( 	COLON
			( assignment )+
		)?
	)
	;
	

exceptionHandler
    :    #("catch" arg_decl action )
    ;


element returns [StateCluster g=null]
{ StringTemplate pred = null; }
    :
	g = elementItem (ast_suffix )?
	|	g=ebnf
	|	g=treeMatcher ( BANG )?
	|	attrLabel
		(	g = elementItem
			|
				g = baseebnf
			|
				g = treeMatcher
			|
				attributeElement
		)
	|	attributeElement
	|	#( SYNPRED block )
	|	action
	|	pred = p:sempred 
		{g = factory.build_SemanticPredicate(p, pred);}
	|	spred:SYN_SEMPRED {g = factory.build_SemanticPredicate(#spred);}
	|	EPSILON {g = factory.build_Epsilon();}
	;


// Templated actions
action
	:
	#(	RCURLY
		bareTemplate
	)
	;

sempred
returns [ StringTemplate code = null ]
{	StringTemplate pred = null;
}
	:
	#(	s:SEMPRED
		pred = template
		{	code = templates.getInstanceOf("validateSemanticPredicate");
		        code.setAttribute("pred", pred.toString());
		}
	)
	;



predicateConstructor
	:
	#(	SEMPRED
		bareTemplate
	)
	;


template
returns [ StringTemplate code = null ]
{ StringTemplate attr = null; }
	:
	(	s:STRING_LITERAL		// anonymous StringTemplate
		{ code = new StringTemplate(userTemplates, #s.getText()); }
		|
			i:ID		// StringTemplate name
			{ code = userTemplates.getInstanceOf(#i.getText()); }
	)
	(
		#(	v:ID
			attr = attributeTemplate
			{ code.setAttribute(#v.getText(), attr); }
		)
	)*
	;


attributeTemplate
returns [StringTemplate index = null; ]
{	StringTemplate tail = null;
	String name = null;
}
	:
	a:ATTRIBUTE
	{	name = #a.getText().substring(1);
		index = templates.getInstanceOf("qualifiedAttribute");
	}
	(	tail = arrayQualifierTemplate[name]
		|
		{	tail = templates.getInstanceOf("Attribute");
			tail.setAttribute("name", getUname(name));
			index.setAttribute("index", tail);
		}
	)
	(	tail = idQualifierTemplate
		{ index.setAttribute("index", tail); }
	)*
	;


idQualifierTemplate
returns [ StringTemplate code = null ]
{	StringTemplate tail = null;
	StringTemplate index = null;
	String name = null;
}
	:
	d:DOT_TEXT
	{ name = #d.getText().substring(1); }
	(	code = arrayQualifierTemplate[name]
		|
			{	code = templates.getInstanceOf("Attribute");
				code.setAttribute("name", getUname(name));
			}
	)
	;

arrayQualifierTemplate[String name]
returns [ StringTemplate code = null ]
{ StringTemplate index = null; }
	:
	#(	LBRACKET
		{	code = templates.getInstanceOf("TableEntry");
			code.setAttribute("name", getUname(name));
		}		
		(	s:STRING_LITERAL
			{ code.setAttribute("contents", s); }
			|
				index = qualifiedIdTemplate
				{ code.setAttribute("contents", index); }
		)
	)
	;


qualifiedIdTemplate
returns [StringTemplate index = null; ]
{ StringTemplate tail = null; }
	:
	i:ID
	{	index = templates.getInstanceOf("qualifiedAttribute");
	}
	(	tail = arrayQualifierTemplate[#i.getText()]
		{ index.setAttribute("index", tail); }
		|
			{ index.setAttribute("index", #i.getText()); }
	)
	(	tail = idQualifierTemplate
		{ index.setAttribute("index", tail); }
	)*
	;



bareTemplate
	:
	(	STRING_LITERAL		// anonymous StringTemplate
		|
			ID		// StringTemplate name
	)
	(
		( assignment )+
	)?
	;


assignment
	:
	#(	ID
		(	attribute
			|
				TOKEN_REF	// symbolic constants
			|
				STRING_LITERAL
		)
	)
	;


attrLabel
	:
	(	// assign but do not remove AST
		a:LABEL_ATTRIBUTE
	|	// assign and remove AST
		ASSIGN_ATTRIBUTE
	)
	( arrayQualifier )?
	(
		idQualifier
	)*
	;

ast_suffix
	:
	CARET
	|	BANG
	;


elementItem
returns [StateCluster g=null]
	:
	g=atom
    |   #(  n:NOT
            (  #( c:KEYWORD (ast1:ast_suffix)? )
	           {
	            int ttype=0;
     			if ( grammar.type==Grammar.LEXER ) {
        			ttype = Grammar.getCharValueFromGrammarCharLiteral(#c.getText());
     			}
     			else {
        			ttype = grammar.getTokenType(#c.getText());
        		}
                IntSet notAtom = grammar.complement(ttype);
                if ( notAtom.isNil() ) {
                    ErrorManager.grammarError(ErrorManager.MSG_EMPTY_COMPLEMENT,
					  			              grammar,
								              #c.token,
									          #c.getText());
                }
	            g=factory.build_Set(notAtom);
	           }
            |  #( t:TOKEN_REF (ast3:ast_suffix)? )
	           {
	           int ttype = grammar.getTokenType(t.getText());
               IntSet notAtom = grammar.complement(ttype);
               if ( notAtom.isNil() ) {
                  ErrorManager.grammarError(ErrorManager.MSG_EMPTY_COMPLEMENT,
				  			              grammar,
							              #t.token,
								          #t.getText());
               }
	           g=factory.build_Set(notAtom);
	           }
            |  g=set
	           {
	           GrammarAST stNode = (GrammarAST)n.getFirstChild();
               IntSet notSet = grammar.complement(stNode.getSetValue());
               stNode.setSetValue(notSet);
               if ( notSet.isNil() ) {
                  ErrorManager.grammarError(ErrorManager.MSG_EMPTY_COMPLEMENT,
				  			              grammar,
							              #n.token);
               }
	           g=factory.build_Set(notSet);
	           }
            )
        	{#n.followingNFAState = g.right;}
         )
    |   #(RANGE a:atom b:atom)
        {g = factory.build_Range(grammar.getTokenType(#a.getText()),
                                 grammar.getTokenType(#b.getText()));}
    |   #(CHAR_RANGE c1:KEYWORD c2:KEYWORD)
        {
        if ( grammar.type==Grammar.LEXER ) {
        	g = factory.build_CharRange(#c1.getText(), #c2.getText());
        }
        }
	;


attribute
	:
	ATTRIBUTE
	( arrayQualifier )?
	( idQualifier )*
	;


ebnf returns [StateCluster g = null]
	:
	#( BANGEDUP g = baseebnf )
	|
		g = baseebnf
	;

baseebnf returns [StateCluster g=null]
{
    StateCluster b = null;
}
    :   #( BLOCK b=block EOB )
        {
        // track decision if > 1 alts
        if ( grammar.getNumberOfAltsForDecisionNFA(b.left)>1 ) {
            b.left.setDescription(grammar.grammarTreeToString((GrammarAST) #BLOCK));
            b.left.setDecisionASTNode((GrammarAST) #BLOCK);
            int d = grammar.assignDecisionNumber( b.left );
            grammar.setDecisionNFA( d, b.left );
            grammar.setDecisionBlockAST(d, #BLOCK);
        }
        g = b;
        }
    |   #( OPTIONAL #( blk:BLOCK b=block EOB ) )
        {
        g = factory.build_Aoptional(b);
    	g.left.setDescription(grammar.grammarTreeToString(#baseebnf,false));
        // there is always at least one alt even if block has just 1 alt
        int d = grammar.assignDecisionNumber( g.left );
		grammar.setDecisionNFA(d, g.left);
        grammar.setDecisionBlockAST(d, (GrammarAST) #blk);
        g.left.setDecisionASTNode(#baseebnf);
    	}
    |   #( CLOSURE #( BLOCK b=block eob:EOB ) )
        {
        g = factory.build_Astar(b);
		// track the loop back / exit decision point
    	b.right.setDescription("()* loopback of "+grammar.grammarTreeToString(#baseebnf,false));
        int d = grammar.assignDecisionNumber( b.right );
		grammar.setDecisionNFA(d, b.right);
        grammar.setDecisionBlockAST(d, #BLOCK);
        b.right.setDecisionASTNode(#eob);
        // make block entry state also have same decision for interpreting grammar
        NFAState altBlockState = (NFAState)g.left.transition(0).target;
        altBlockState.setDecisionASTNode(#baseebnf);
        altBlockState.setDecisionNumber(d);
        g.left.setDecisionNumber(d); // this is the bypass decision (2 alts)
        g.left.setDecisionASTNode(#baseebnf);
    	}
    |   #( POSITIVE_CLOSURE #( blk2:BLOCK b=block eob3:EOB ) )
        {
        g = factory.build_Aplus(b);
        // don't make a decision on left edge, can reuse loop end decision
		// track the loop back / exit decision point
    	b.right.setDescription("()+ loopback of "+grammar.grammarTreeToString(#baseebnf,false));
        int d = grammar.assignDecisionNumber( b.right );
		grammar.setDecisionNFA(d, b.right);
	grammar.setDecisionBlockAST(d, #blk2);
        b.right.setDecisionASTNode(#eob3);
        // make block entry state also have same decision for interpreting grammar
        NFAState altBlockState = (NFAState)g.left.transition(0).target;
        altBlockState.setDecisionASTNode(#baseebnf);
        altBlockState.setDecisionNumber(d);
        }
    ;

treeMatcher returns [StateCluster g=null]
{
StateCluster e=null;
}
	:   #( TREE_BEGIN
		   {GrammarAST el=(GrammarAST)_t;}
		   g=element
		   {
           StateCluster down = factory.build_Atom(Label.DOWN);
           // TODO set following states for imaginary nodes?
           //el.followingNFAState = down.right;
		   g = factory.build_AB(g,down);
		   }
		   ( {el=(GrammarAST)_t;} e=element {g = factory.build_AB(g,e);} )*
		   {
           StateCluster up = factory.build_Atom(Label.UP);
           //el.followingNFAState = up.right;
		   g = factory.build_AB(g,up);
		   // tree roots point at right edge of DOWN for LOOK computation later
		   #treeMatcher.NFATreeDownState = down.left;
		   }
		 )
    ;

atom returns [StateCluster g=null]
    :   #( r:RULE_REF (rarg:ARG_ACTION)? (as1:ast_suffix)? )
        {
        NFAState start = grammar.getRuleStartState(r.getText());
        if ( start!=null ) {
            int ruleIndex = grammar.getRuleIndex(r.getText());
            g = factory.build_RuleRef(ruleIndex, start);
            r.followingNFAState = g.right;
            if ( g.left.transition(0) instanceof RuleClosureTransition
            	 && grammar.type!=Grammar.LEXER )
            {
                addFollowTransition(r.getText(), g.right);
            }
            // else rule ref got inlined to a set
        }
        }

	|   t:TOKEN_REF
        {
        if ( grammar.type==Grammar.LEXER ) {
            NFAState start = grammar.getRuleStartState(t.getText());
            if ( start!=null ) {
                int ruleIndex = grammar.getRuleIndex(t.getText());
                g = factory.build_RuleRef(ruleIndex, start);
                // don't add FOLLOW transitions in the lexer;
                // only exact context should be used.
            }
        }
        else {
            int tokenType = grammar.getTokenType(t.getText());
            g = factory.build_Atom(tokenType);
            t.followingNFAState = g.right;
        }
        }

	|   c:KEYWORD
    	{
    	if ( grammar.type==Grammar.LEXER ) {
    		g = factory.build_KeywordAtom(c.getText());
    	}
    	else {
            int tokenType = grammar.getTokenType(c.getText());
            g = factory.build_Atom(tokenType);
            c.followingNFAState = g.right;
    	}
    	}

	|   s:STRING_LITERAL
    	{
     	if ( grammar.type==Grammar.LEXER ) {
     		g = factory.build_StringLiteralAtom(s.getText());
     	}
     	else {
             int tokenType = grammar.getTokenType(s.getText());
             g = factory.build_Atom(tokenType);
             s.followingNFAState = g.right;
     	}
     	}

	|   WILDCARD         {g = factory.build_Wildcard();}

	|	g=set
	;


set returns [StateCluster g=null]
{
	IntSet elements=new IntervalSet();
	#set.setSetValue(elements); // track set for use by code gen
}
	:
	#(s:SET (setElement[elements])+)
        {
        	g = factory.build_Set(elements);
	        #s.followingNFAState = g.right;
        }
		//{System.out.println("set elements="+elements.toString(grammar));}
	;


setElement[IntSet elements]
{
    int ttype;
}
    :   c:KEYWORD
        {
     	if ( grammar.type==Grammar.LEXER ) {
        	ttype = Grammar.getCharValueFromGrammarCharLiteral(c.getText());
     	}
     	else {
        	ttype = grammar.getTokenType(c.getText());
        }
        if ( elements.member(ttype) ) {
			ErrorManager.grammarError(ErrorManager.MSG_DUPLICATE_SET_ENTRY,
									  grammar,
									  #c.token,
									  #c.getText());
        }
        elements.add(ttype);
        }
    |   t:TOKEN_REF
        {
        ttype = grammar.getTokenType(t.getText());
        if ( elements.member(ttype) ) {
			ErrorManager.grammarError(ErrorManager.MSG_DUPLICATE_SET_ENTRY,
									  grammar,
									  #t.token,
									  #t.getText());
        }
        elements.add(ttype);
        }
    |   s:STRING_LITERAL
        {
        ttype = grammar.getTokenType(s.getText());
        if ( elements.member(ttype) ) {
			ErrorManager.grammarError(ErrorManager.MSG_DUPLICATE_SET_ENTRY,
									  grammar,
									  #s.token,
									  #s.getText());
        }
        elements.add(ttype);
        }
    |	#(CHAR_RANGE c1:KEYWORD c2:KEYWORD)
    	{
     	if ( grammar.type==Grammar.LEXER ) {
	        int a = Grammar.getCharValueFromGrammarCharLiteral(c1.getText());
    	    int b = Grammar.getCharValueFromGrammarCharLiteral(c2.getText());
    		elements.addAll(IntervalSet.of(a,b));
     	}
    	}
    ;

// ATTRIBUTE syntax
attributeGroup
	:
	#( 
		ATTRIBUTE_GROUP
		(
			attributeItem
			|
				STRING_LITERAL
		)*
	)
	;

attributeBlock
	:
	(
		attributeItem
	)*
	|
		predicateConstructor
		(
			attributeItem
		)*
		( 
			OR
			attributeBlock
		)
	;


attributeStatements
	:
	#(	ATTRIBUTE_STMT
		attributeBlock
	)
	;


attributeItem
	:
	( attrLabel )?
	attributeElement
	;

attributeElement
	:
	attribute
	(
		CARET
	)?
	|
		attributeGroup
		( CARET )?
	|
		attributeStatements
	|
		CURRENT_TREE
		( BANG )?
	|
		LPAREN
		(
			attributeItem
		)*
		RPAREN
	|
		treeConstructor
	;


treeConstructor
	:
	#(
		TREE_CONSTRUCTOR
		attributeItem
		(
			attributeItem
		)+
	)
	;
// END ATTRIBUTE syntax

