header {
/*
 [The "BSD licence"]
 Copyright (c) 2005-2006 Terence Parr
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
	package org.antlr_yggdrasil.generated;
	import org.antlr_yggdrasil.tool.*;
	import org.antlr_yggdrasil.tool.Grammar;
	import java.util.*;
}

class YggdrasilDefineGrammarItems extends TreeParser;

options {
	importVocab = Yggdrasil;
	ASTLabelType = "GrammarAST";
    codeGenBitsetTestThreshold=999;
}

{
protected Grammar grammar;
protected GrammarAST root;
protected String currentRuleName;
protected int outerAltNum = 0;
protected int blockLevel = 0;

    public void reportError(RecognitionException ex) {
		Token token = null;
		if ( ex instanceof MismatchedTokenException ) {
			token = ((MismatchedTokenException)ex).token;
		}
		else if ( ex instanceof NoViableAltException ) {
			token = ((NoViableAltException)ex).token;
		}
        ErrorManager.syntaxError(
            ErrorManager.MSG_SYNTAX_ERROR,
            token,
            "define: "+ex.toString(),
            ex);
    }

	protected void finish() {
		trimGrammar();
	}

	/** Remove any lexer rules from a COMBINED; already passed to lexer */
	protected void trimGrammar() {
		if ( grammar.type!=Grammar.COMBINED ) {
			return;
		}
		// form is (header ... ) ( grammar ID (scope ...) ... ( rule ... ) ( rule ... ) ... )
		GrammarAST p = root;
		// find the grammar spec
		while ( !p.getText().equals("grammar") ) {
			p = (GrammarAST)p.getNextSibling();
		}
		p = (GrammarAST)p.getFirstChild(); // jump down to first child of grammar
		// look for first RULE def
		GrammarAST prev = p; // points to the ID (grammar name)
		while ( p.getType()!=RULE ) {
			prev = p;
			p = (GrammarAST)p.getNextSibling();
		}
		// prev points at last node before first rule subtree at this point
		while ( p!=null ) {
			String ruleName = p.getFirstChild().getText();
			//System.out.println("rule "+ruleName+" prev="+prev.getText());
			if ( Character.isUpperCase(ruleName.charAt(0)) ) {
				// remove lexer rule
				prev.setNextSibling(p.getNextSibling());
			}
			else {
				prev = p; // non-lexer rule; move on
			}
			p = (GrammarAST)p.getNextSibling();
		}
		//System.out.println("root after removal is: "+root.toStringList());
	}
}



file[Grammar g]
	:
	(
		fileheader
	)*
	(
		typeDecls
	)?
	(
		optionsSpec
	)?
	grammar[g]
	;


fileheader
	:
	#(	"header"
		( ID )?
		LCURLY
		( optionsSpec )?
		( tokensSpec )?
		( action )?
	)
	;


// Attribute specification syntax
grammarAttributeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			grammarAttributeDecl
		)*
	)
	;

grammarAttributeDecl
	:
	(
		"public"
	)?
	attributeVarDecl
	|
		"import"
		STRING_LITERAL
	;

ruleAttributeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			attributeVarDecl
			|
				attributeUseDecl
		)*
	)
	;


typeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			attributeTypeDecl
		)*
	)
	;
	

attributeTypeDecl
	:
	#( 
		"atomic"
		"native"
		ID
	)
	|
		#( 
			LCURLY
			(
				"native"
			)?
			ID
			ID
			(
				attributeVarDecl
			)*
		)
	;

attributeVarDecl
	:
	#( 
		SEMI
		ID
		(	OPEN_ELEMENT_OPTION
			ID ( ID )*
			CLOSE_ELEMENT_OPTION
		)?
		ID
	)
	;


idQualifier
	:
	DOT_TEXT
	( arrayQualifier )?
	;

arrayQualifier
	:
	#(	LBRACKET
		(	STRING_LITERAL
			|
				ID
				( arrayQualifier )?
				(idQualifier)*
		)
	)
	;


attributeUseDecl
	:
	ID
	ASSIGN
	(
		ID
		( arrayQualifier )?
		( idQualifier )*
	)
	|
		"new"
		ID
	|
		"using"
		ID
	;
// END ATTRIBUTE DECLARATION and TYPE DEFINITION


grammar[Grammar g]
{
grammar = g;
root = #grammar;
}
    :   ( #( LEXER_GRAMMAR 	  {grammar.type = Grammar.LEXER;} 	    grammarSpec )
	    | #( PARSER_GRAMMAR   {grammar.type = Grammar.PARSER;}      grammarSpec )
	    | #( TREE_GRAMMAR     {grammar.type = Grammar.TREE_PARSER;} grammarSpec )
	    | #( COMBINED_GRAMMAR {grammar.type = Grammar.COMBINED;}    grammarSpec )
	    )
	    {finish();}
    ;


grammarSpec
{
Map opts=null;
Token optionsStartToken=null;
}
	:
	id:ID
	(cmt:DOC_COMMENT)?
	(
		grammarAttributeDecls
	)?

        ( {optionsStartToken=((GrammarAST)_t).getToken();}
          optionsSpec
        )?
        (tokensSpec)?
        rules
	;


optionsSpec
	:	OPTIONS
	;

/*
optionsSpec returns [Map opts=new HashMap()]
    :   #( OPTIONS (option[opts])+ )
    ;

option[Map opts]
{
    String key=null;
    Object value=null;
}
    :   #( ASSIGN id:ID {key=#id.getText();} value=optionValue )
        {opts.put(key,value);}
    ;

optionValue returns [Object value=null]
    :   id:ID			 {value = #id.getText();}
    |   s:STRING_LITERAL {String vs = #s.getText();
                          value=vs.substring(1,vs.length()-1);}
    |   c:KEYWORD   {String vs = #c.getText();
                          value=vs.substring(1,vs.length()-1);}
    |   i:INT            {value = new Integer(#i.getText());}
//  |   cs:charSet       {value = #cs;} // return set AST in this case
    ;

charSet
	:   #( CHARSET charSetElement )
	;

charSetElement
	:   c:KEYWORD
	|   #( OR c1:KEYWORD c2:KEYWORD )
	|   #( RANGE c3:KEYWORD c4:KEYWORD )
	;
*/

tokensSpec
	:	#( TOKENS ( tokenSpec )+ )
	;

tokenSpec
	:
	t:TOKEN_REF
	( OPEN_ELEMENT_OPTION ID )?
	|
		#( ASSIGN
			t2:TOKEN_REF
			( OPEN_ELEMENT_OPTION ID )?
			KEYWORD
		 )
	;

rules
	:
	(
		rule
		|
			constructorRule
	)+
	;

rule
{
	String mod=null;
	String name=null;
	Rule r = null;
}
	:
	#(	RULE id:ID
		(mod=modifier)?
		(optionsSpec)?
		{	name = #id.getText();
			currentRuleName = name;
			if ( Character.isUpperCase(name.charAt(0)) &&
					 grammar.type==Grammar.COMBINED )
			{
			// a merged grammar spec, track lexer rules and send to another grammar
				grammar.defineLexerRuleFoundInParser(#id.getToken(), #rule);
			}
			else {
				int numAlts = countAltsForRule(#rule);
				grammar.defineRule(#id.getToken(), mod, null, #rule, null, numAlts);
				r = grammar.getRule(name);
			}
		}
		( ruleAttributeDecls )?
		( action )?
		{this.blockLevel=0;}
		b:block
		(exceptionGroup)?
		EOR
	)
    ;


// Tree construction rule
constructorRule
	:
	#(	ATTRIBUTE_COLON
		RULE_REF
		( attributeItem )+
	)
	;


countAltsForRule returns [int n=0]
	:
	#(	RULE id:ID
		(modifier)?
		(optionsSpec)?
		(ruleAttributeDecls)?
		(action)?
	       	#(	BLOCK
			(optionsSpec)?
			(ALT {n++;})+
			EOB
		)
		(exceptionGroup)?
		EOR
         )
	;


modifier returns [String mod]
{
mod = #modifier.getText();
}
	:	"protected"
	|	"public"
	|	"private"
	|	"fragment"
	;


block
{
this.blockLevel++;
if ( this.blockLevel==1 ) {this.outerAltNum=1;}
}
    :   #(  BLOCK
            (optionsSpec)?
            ( alternative 
              {if ( this.blockLevel==1 ) {this.outerAltNum++;}}
            )+
            EOB
         )
         {this.blockLevel--;}
    ;


alternative
    :   #( ALT (element)+ EOA )
    ;

exceptionGroup
	:	( exceptionSpec )+
    ;


arg_decl
	:
	#(	RBRACKET
		(	STRING_LITERAL		// anonymous StringTemplate
			|
				ID		// StringTemplate name
		)
		( 	COLON
			( assignment )+
		)?
	)
	;
	

exceptionSpec
    :   #("exception" ( arg_decl )? ( exceptionHandler )+ )
    ;

exceptionHandler
    :    #("catch" arg_decl action)
    ;


element
    :
	(	atom
		|	#(NOT atom)
		|	#(RANGE atom atom)
		|	#(CHAR_RANGE atom atom)
	)
	( ast_suffix )?
 	|   ebnf
	|   treeMatcher		( BANG )?
	|
		l:attrLabel
		(	atom
			|
				#(NOT atom)
			|
				baseebnf
			|
				treeMatcher
			|
				attributeElement
		)
	|	attributeElement
	|	#( SYNPRED block )
	|	action
	|	sempred
	|	SYN_SEMPRED
	|	EPSILON 
    ;

// Templated actions
action
	:
	#(	RCURLY
		template
	)
	;


sempred
	:
	#(	SEMPRED
		template
	)
	;


template
	:
	(	STRING_LITERAL		// anonymous StringTemplate
		|
			ID		// StringTemplate name
	)
	(
		( assignment )+
	)?
	;


assignment
	:
	#(	ID
		(	attribute
			|
				TOKEN_REF	// symbolic constants
			|
				STRING_LITERAL
		)
	)
	;


attrLabel
	:
	(	// assign but do not remove AST
		a:LABEL_ATTRIBUTE
	|	// assign and remove AST
		ASSIGN_ATTRIBUTE
	)
	( arrayQualifier )?
	(
		idQualifier
	)*
	;

attribute
	:
	ATTRIBUTE
	( arrayQualifier )?
	( idQualifier )*
	;


ebnf
	:
	#( BANGEDUP baseebnf )
	|
		baseebnf
	;


baseebnf:   (dotLoop)=> dotLoop // .* or .+
    |   block
    |   #( OPTIONAL block )
    |   #( CLOSURE block )
    |   #( POSITIVE_CLOSURE block )
    ;

/** Track the .* and .+ idioms and make them greedy by default.
 *  If someone specifies an option, it won't match these
 */
dotLoop
{
    GrammarAST block = (GrammarAST)#dotLoop.getFirstChild();
}
    :   (   #( CLOSURE dotBlock )           
        |   #( POSITIVE_CLOSURE dotBlock )
        )
        {
        Map opts=new HashMap();
        opts.put("greedy", "false");
        if ( grammar.type!=Grammar.LEXER ) {
            // parser grammars assume k=1 for .* loops
            // otherwise they look til EOF!
            opts.put("k", new Integer(1));
        }
        block.setOptions(grammar,opts);
        }
    ;

dotBlock
    :   #( BLOCK #( ALT WILDCARD EOA ) EOB )
    ;

treeMatcher:   #(TREE_BEGIN element (element)*)
    ;

atom
    :   r:RULE_REF
    	{grammar.altReferencesRule(currentRuleName, #r, this.outerAltNum);}
    |   t:TOKEN_REF
    	{
    	if ( grammar.type==Grammar.LEXER ) {
    		grammar.altReferencesRule(currentRuleName, #t, this.outerAltNum);
    	}
    	else {
    		grammar.altReferencesTokenID(currentRuleName, #t, this.outerAltNum);
    	}
    	}
    |   c:KEYWORD
    	{
    	if ( grammar.type!=Grammar.LEXER ) {
    		Rule rule = grammar.getRule(currentRuleName);
			if ( rule!=null ) {
				rule.trackTokenReferenceInAlt(#c, outerAltNum);
    		}
    	}
    	}
    |   s:STRING_LITERAL
    	{
    	if ( grammar.type!=Grammar.LEXER ) {
    		Rule rule = grammar.getRule(currentRuleName);
			if ( rule!=null ) {
				rule.trackTokenReferenceInAlt(#s, outerAltNum);
    		}
    	}
    	}
    |   WILDCARD
    |	set
    ;

ast_suffix
	:	CARET
	|	BANG
	;

set :   #(SET (setElement)+ (ast_suffix)? )
    ;

setElement
    :   c:KEYWORD
    |   t:TOKEN_REF
    |   s:STRING_LITERAL
    |	#(CHAR_RANGE c1:KEYWORD c2:KEYWORD)
    ;

// ATTRIBUTE syntax
attributeGroup
	:
	#( 
		ATTRIBUTE_GROUP
		(
			attributeItem
			|
				STRING_LITERAL
		)*
	)
	;

attributeBlock
	:
	(
		attributeItem
	)*
	|
		sempred
		(
			attributeItem
		)*
		( 
			OR
			attributeBlock
		)
	;


attributeStatements
	:
	#(	ATTRIBUTE_STMT
		attributeBlock
	)
	;


attributeItem
	:
	( attrLabel )?
	attributeElement
	;

attributeElement
	:
	attribute
	(
		CARET
	)?
	|
		attributeGroup
		( CARET )?
	|
		attributeStatements
	|
		CURRENT_TREE
		( BANG )?
	|
		LPAREN
		(
			attributeItem
		)*
		RPAREN
	|
		treeConstructor
	;


treeConstructor
	:
	#(
		TREE_CONSTRUCTOR
		attributeItem
		(
			attributeItem
		)+
	)
	;
// END ATTRIBUTE syntax

