/*
 [The "BSD licence"]
 Copyright (c) 2005-2006 Terence Parr, Loring Craymer
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.antlr_yggdrasil.runtime;

import java.util.List;
import java.util.ArrayList;
import java.util.Stack;

/** A parser for TokenStreams.  "parser grammars" result in a subclass
 *  of this.
 */
public class Parser extends BaseRecognizer {
	protected TreeBuilder builder = new TreeBuilder();
    protected TokenStream input;
    private Carrier tree;

	public Parser(TokenStream input) {
        setTokenStream(input);
    }
	
	public void reset() {
		super.reset();
		builder.reset();
		tree = null;
	}

	/** Set the token stream and reset the parser */
	public void setTokenStream(TokenStream input) {
		this.input = input;
		reset();
	}

    public TokenStream getTokenStream() {
		return input;
	}

	public void setBuilder(TreeBuilder builder) {
		this.builder = builder;
	}

	public TreeBuilder getBuilder() {
		return builder;
	}
	
	public Carrier getTree() {
		if (tree == null) {
			int len = builder.getLen();
			if (len > 1) {
				builder.processFromMark();
				tree = (Carrier) builder.pop();
			}
			else if (len == 1) {
				Object o = builder.pop();
				if (o instanceof Carrier)
					tree = (Carrier) o;
				else
					tree = ((Payload) o).wrap();
			}
			else
				return null;
		}
		return tree;
	}
	
	public void setTree(Carrier t) {
		tree = t;
	}
	
	public PayloadFactory getFactory() {
		return factory;
	}
}
