/*
 [The "BSD licence"]
 Copyright (c) 2006 Loring Craymer
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// Test grammar for automated tree grammar production

//header {
//}


@{
	Payload PayloadToken {
		Integer line;
		Integer charPositionInLine;
		Integer channel;
		Integer index;
		Integer start;
		Integer stop;
	}

	PayloadToken Payload1 {
		Integer foo;
	}

	PayloadToken Payload2 {
		Integer bar;
	}
}

options {
	package="examples.generic.treetest.*";
}


parser grammar TestParser;

@{
	Payload a;
	Payload b;
	PayloadToken c;
	PayloadToken d;
	PayloadToken e;
	PayloadToken f;
	PayloadToken g;
	PayloadToken h;
	PayloadToken i;
	PayloadToken j;
	PayloadToken k;
	PayloadToken v;
	PayloadToken y;
	Carrier testCarrier;
	Table<String, PayloadToken> table;
	Queue<PayloadToken> queue;
}

options {
	k=2;
	tokenVocab="TEST";
//	buildAST = true;
	userTemplates = "user";
}

tokens {
	A<Payload1>;
	B<PayloadToken>;
	C<Payload2>;
	F<PayloadToken>;
}

topLevel
	:
	( topAlt) +
	;

topAlt
	:
	insert
	|	insertTree
	|	insertNode
	|	insertRootNode
	|	insertTrees
	|	retTest
	|	cutTest
	|	treeGen
	|	ebnfTest
	|	groupTest
	|	constructionPredTest
	|	tableTest
	|	reverseQTest
	|	queueItemTest
	;
	

insert
	:
	@a = A
	@b = B
	C D  @a @b  E
	;

insertTree
	:
	@f = F
	@g = G
	H I  ^( @f @g )
	;

insertNode
	:
	J K @( "L" "L") M N
	;

insertRootNode
	:
	O P @( "Q" "Q")^ R S
	;


insertTrees
	:
	@a = Aa
	@b = Ba
	@c = Ca
	@d = Da
	@e = Ea
	f = Fa
	@g = Ga
	@h = Ha
	@i = Ia
	@j = Ja
	@k = Ka
	^( @a ^( @b @c ^( @d @e @f ) @g ^( @h @i @j ) ) ) @k
	;

retTest
	:
	T U @v = V @$!  @v
	;

cutTest
	:
	W X @y = Y  @$!  Z A
	;


treeGen
	:
	a_1 b_1 ( Cb^ | Db^ ) e_1 f_1
	;

a_1
	:
	Ab Bb^ Eb
	;

b_1
	:
	Fb Gb^ Hb Ib^ Jb
	;

e_1
	:
	Kb^ Lb Mb Nb^ Ob^
	;

f_1
	:
	Pb Qb Rb
	;

/*
notTest
	:
	~(A | B)
	@a = ~C
	D
	@a
	;
*/

ebnfTest
	:
	Ac
	Bc^
	@testCarrier = ( Dc Ec Fc )
	Gc
	@testCarrier
	Hc
	;

groupTest
	:
	@a = Ic
	@b = Jc
	c = @( @a @b )
	;


/*
predTest
@{	using a;
//	new b;
	c = d;
}
	:
	{ textPred: string = @a.text; }? Mc
	| Nc
	;
*/


constructionPredTest
	:
	@a = Oc
	@b = Pc
	@c = Qc
	@d = Rc
	@e = Sc
	@{	{ testCpred: var = @a; value = "Pc";}? @c
		|
			{ testCpred: var = @b; value = "Pc"; }? @d
		|
			@e
	}
	;

tableTest
	:
	@table = @( )	// initialize
	@table["test"] = B
	C D E
	@table["test"]
	F
	;

reverseQTest
	:
	@queue = @( )
	@queue.first = G
	@queue.first = H
	@queue.first = I
	@queue.first = J
	@queue
	;

queueItemTest
	:
	@queue = @( )
	@queue.last = K
	@queue.last = L
	@queue.last = M
	@queue.last = N
	@queue.first
	@queue.last
	@queue.first
	@queue.last
	;
	
